/*
Schreibt ein Wort in die Luft

  Realisiert mit ATmega8 (8MHz interner RC)
  7 LEDs an PortB0..B5 und PortD7
  Ausgang vom Beschleunigungssensor MMA2260D an Pin 28 (ADC5)    
*/
    
#define F_CPU 8000000UL	/* CPU clock in Hertz */

#include <inttypes.h>
#include <avr/io.h>
#include <stdio.h>
#include <math.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define U8 unsigned char
#define I16 int
#define I32 long int

char led1[]=" $$$$$$$  $$$$  $$$$$   $$$$$ $     $  ";
char led2[]="      $  $    $  $   $  $      $   $   "; 
char led3[]="     $   $    $  $   $  $       $ $    ";
char led4[]="   $$$   $$$$$$  $$$$   $$$      $     ";
char led5[]="   $     $    $  $   $  $       $ $    ";
char led6[]="  $      $    $  $   $  $      $   $   ";
char led7[]=" $$$$$$$ $    $ $$$$$   $$$$$ $     $  ";

#define SPALTEN 38

//--------------------------------------------------------------------------------------
int tiefpassErde(int wert){
static long summe;
  //Mittelwert ber 5000 Werte
  summe += ((long)wert*65536 - summe)/512;
  return summe / 65536;
}

//--------------------------------------------------------------------------------------
int tiefpassBein(int wert){
static long summe;
  //Mittelwert ber 250 Werte
  summe += ((long)wert*65536 - summe)/32;
  return summe / 65536;
}

//--------------------------------------------------------------------------------------
int main(void){
  U8 spalte;   
  U8 ledsB;  
  U8 ledsD; 
  U8 i;
  I16 n,a, delta;
  I32 bein,erde; 

  DDRB=0xFF;
  DDRD=0xFF;
  PORTB=0;   
  PORTD=0;  
  ADMUX=5+64; //Uref=AVcc 
  ADCSRA=128+64+32 + 7; //Free running, Teiler 1:128 = 62500
    
  erde=512;  
  bein=512;  
  while(1){

    do{
      _delay_us(300);
      a=ADC; //0..1023
      erde = tiefpassErde(a);
      bein = tiefpassBein(a);
      delta = erde -bein;
    }while (delta < 90);
    _delay_ms(100);

    for (spalte=0; spalte < SPALTEN; spalte++){
      for (i=0; i<(7-spalte/9); i++){
        _delay_us(40);
        a=ADC; //0..1023
        erde = tiefpassErde(a);
        bein = tiefpassBein(a);
      }

      ledsB=0;  
      ledsD=0;  
      if (' '!=led1[spalte] ) ledsB|=1;
      if (' '!=led2[spalte] ) ledsB|=2;
      if (' '!=led3[spalte] ) ledsB|=4;
      if (' '!=led4[spalte] ) ledsB|=8;
      if (' '!=led5[spalte] ) ledsB|=16;
      if (' '!=led6[spalte] ) ledsB|=32;
      
      if (' '!=led7[spalte] ) ledsD|=128;
      PORTB=ledsB;    
      PORTD=ledsD;    
    }        

    for (n=0; n<1000; n++){
      _delay_us(300);
      a=ADC; //0..1023
      erde = tiefpassErde(a);
      bein = tiefpassBein(a);
    }

  }      
  return 0;    
}
